SystemOrganization addCategory: #'CT-PolySource-Events'!
SystemOrganization addCategory: #'CT-PolySource-Snapshots'!
SystemOrganization addCategory: #'CT-PolySource-Tests'!

PSSystemEventProcessor subclass: #CtPsMarkupEventProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Events'!

!CtPsMarkupEventProcessor methodsFor: 'testing' stamp: 'mt 10/4/2010 15:30:42.881'!
canProcess: aSystemEvent with: aSourceCode

	^ aSystemEvent itemMarkup targets anySatisfy: [:target |
		target isMethodSource and: [aSourceCode packages includes: target package name]].! !

!CtPsMarkupEventProcessor methodsFor: 'execution' stamp: 'mt 10/4/2010 15:35:49.197'!
execute: aSystemEvent

	aSystemEvent isAdded
		ifTrue: [SystemMarkups addMarkup: aSystemEvent itemMarkup].
		
	aSystemEvent isModified
		ifTrue: [SystemMarkups addMarkup: aSystemEvent itemMarkup]. "What about the #oldItem here?"
		
	aSystemEvent isRemoved
		ifTrue: [SystemMarkups removeMarkup: aSystemEvent itemMarkup].! !

!CtPsMarkupEventProcessor methodsFor: 'accessing' stamp: 'mt 10/4/2010 15:26:02.569'!
supportedItemKinds

	^ #(markup)! !

PSSnapshotBuilder subclass: #CtPsMarkupSnapshotBuilder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Snapshots'!

!CtPsMarkupSnapshotBuilder commentStamp: 'mt 10/4/2010 17:31:16.148' prior: 0!
Builds a snapshot that is a wrapper for the default snapshot with additional markups. See PSMarkupSnapshot class.!

!CtPsMarkupSnapshotBuilder class methodsFor: 'priority' stamp: 'mt 10/4/2010 17:28:30.483'!
priority

	^ 2! !

!CtPsMarkupSnapshotBuilder methodsFor: 'building' stamp: 'mt 10/4/2010 17:36:35.961'!
buildSnapshot: somePackageNames

	| snapshot |
	snapshot := CtPsMarkupSnapshot new.
	snapshot defaultSnapshot: (PSDefaultSnapshotBuilder new buildSnapshot: somePackageNames).
	snapshot markups:
		(somePackageNames collect: [:packageName |
			SystemMarkups createSnapshot: packageName]).
	^ snapshot! !

!CtPsMarkupSnapshotBuilder methodsFor: 'loading' stamp: 'mt 10/4/2010 17:35:08.763'!
loadSnapshot: aSnapshot

	"Load the embedded default snapshot first."
	aSnapshot defaultSnapshot load.
	
	"Now load the markups."
	aSnapshot markups do: [:markupSnapshot |
		SystemMarkups loadSnapshot: markupSnapshot].! !

TestCase subclass: #CtPsEventTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Tests'!

!CtPsEventTests methodsFor: 'tests' stamp: 'mt 10/5/2010 14:04:01.357'!
test01markupAddedEvent

	| markup evt upd |
	markup := CtMarkup new.
	evt := AddedMarkupEvent markup: markup.
	upd := CtPsMarkupEventProcessor new process: evt.
	
	self
		assert: markup equals: upd event item;
		assert: markup equals: upd event itemMarkup.! !

!CtPsEventTests methodsFor: 'tests' stamp: 'mt 10/5/2010 14:03:51.705'!
test02markupModifiedEvent

	| oldMarkup newMarkup evt upd |
	oldMarkup := CtMarkup new.
	newMarkup := oldMarkup copyWithTagAdded: #good.
	evt := ModifiedMarkupEvent from: oldMarkup to: newMarkup.
	upd := CtPsMarkupEventProcessor new process: evt.
	
	self
		assert: newMarkup equals: upd event item;
		assert: newMarkup equals: upd event itemMarkup;
		assert: oldMarkup equals: upd event oldItem.! !

!CtPsEventTests methodsFor: 'tests' stamp: 'mt 10/5/2010 14:06:10.827'!
test03markupRemovedEvent

	| markup evt upd |
	markup := CtMarkup new.
	evt := RemovedMarkupEvent markup: markup.
	upd := CtPsMarkupEventProcessor new process: evt.
	
	self
		assert: markup equals: upd event item;
		assert: markup equals: upd event itemMarkup.! !

TestCase subclass: #CtPsProxyTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Tests'!

!CtPsProxyTests methodsFor: 'running' stamp: 'mt 10/6/2010 12:07:24.354'!
setUp

	super setUp.
	PSSystemDictionaryProxy install.! !

!CtPsProxyTests methodsFor: 'running' stamp: 'mt 10/6/2010 12:07:38.419'!
tearDown

	PSSystemDictionaryProxy uninstall.
	super tearDown.! !

!CtPsProxyTests methodsFor: 'tests' stamp: 'mt 10/6/2010 12:06:06.37'!
test01customMarkups

	| markup |
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self class
			selector: thisContext selector
			from: 1 to: 5);
			next.
	
	self deny: ((SystemMarkups
		markupsForClass: self class
		selector: thisContext selector
		kind: #methodSource) includes: markup).
			
	PSCurrentRemoteClient value: PSRemoteClient new during: [
		self deny: ((SystemMarkups
			markupsForClass: self class
			selector: thisContext selector
			kind: #methodSource) includes: markup).

		SystemMarkups addMarkup: markup.

		self assert: ((SystemMarkups
			markupsForClass: self class
			selector: thisContext selector
			kind: #methodSource) includes: markup)].
		
	self deny: ((SystemMarkups
		markupsForClass: self class
		selector: thisContext selector
		kind: #methodSource) includes: markup).
! !

TestCase subclass: #CtPsSnapshotTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Tests'!

!CtPsSnapshotTests methodsFor: 'support' stamp: 'mt 10/6/2010 11:44:11.177'!
createTestClass

	Object
		subclass: #CtPsTestClass
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'CTP-Temporary'.
		
	self getTestClass
		compile: 'test'
		classified: #test.! !

!CtPsSnapshotTests methodsFor: 'support' stamp: 'mt 10/6/2010 11:43:28.732'!
getTestClass

	^ Smalltalk at: #CtPsTestClass! !

!CtPsSnapshotTests methodsFor: 'support' stamp: 'mt 10/6/2010 11:43:39.208'!
removeTestClass

	self getTestClass removeFromSystem: false.! !

!CtPsSnapshotTests methodsFor: 'running' stamp: 'mt 10/6/2010 11:44:24.961'!
setUp

	super setUp.
	PSSystemDictionaryProxy install.
	self createTestClass.! !

!CtPsSnapshotTests methodsFor: 'running' stamp: 'mt 10/6/2010 11:46:16.391'!
tearDown

	self removeTestClass.
	SystemOrganization removeSystemCategory: #'CTP-Temporary'.
	
	PSSystemDictionaryProxy uninstall.
	super tearDown.! !

!CtPsSnapshotTests methodsFor: 'tests' stamp: 'mt 10/5/2010 14:49:45.621'!
test01updateSnapshot

	| localClient markup |
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self class
			selector: thisContext selector
			from: 1 to: 10);
		next.

	SystemMarkups addMarkup: markup.

	"Creating a local client after adding a markup avoids another manual snapshot update."
	localClient := PSLocalClient new
		disableNotificationProcessing; "Not now."
		connection: PSSqSyncConnectionMock new;
		addPackage: #'CT-PolySource'.
	
	self
		assert: (localClient sourceCode snapshot isKindOf: CtPsMarkupSnapshot);
		assert: markup equals: (localClient sourceCode snapshot markups first activeMarkups at: markup uuid).
	
	SystemMarkups removeMarkup: markup.	! !

!CtPsSnapshotTests methodsFor: 'tests' stamp: 'mt 10/5/2010 14:51:09.27'!
test02addUpdate

	| localClient markup |
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self class
			selector: thisContext selector
			from: 1 to: 10);
		next.
		
	localClient := PSLocalClient new
		enableNotificationProcessing;
		connection: PSSqSyncConnectionMock new;
		addPackage: #'CT-PolySource'.

	"Creating a local client BEFORE adding a markup triggers an update besides the snapshot."
	SystemMarkups addMarkup: markup.
	
	self
		assert: (localClient sourceCode updates last event itemKind = #markup);
		assert: markup equals: localClient sourceCode updates last event item.
	
	SystemMarkups removeMarkup: markup.	! !

!CtPsSnapshotTests methodsFor: 'tests' stamp: 'mt 10/6/2010 11:24:41.451'!
test03processUpdate

	| localClient markup update |
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self class
			selector: thisContext selector
			from: 1 to: 10);
		next.
	update := CtPsMarkupEventProcessor new process: (AddedMarkupEvent markup: markup).
		
	localClient := PSLocalClient new
		enableNotificationProcessing;
		connection: PSSqSyncConnectionMock new;
		addPackage: #'CT-PolySource'.

	self
		deny: ((SystemMarkups
			markupsForClass: self class
			selector: thisContext selector
			kind: #methodSource) includes: markup).

	"Incoming update should be processed locally."
	localClient sourceCode addUpdate: update.
	
	self
		assert: ((SystemMarkups
			markupsForClass: self class
			selector: thisContext selector
			kind: #methodSource) includes: markup).
	
	SystemMarkups removeMarkup: markup.	! !

!CtPsSnapshotTests methodsFor: 'tests' stamp: 'mt 10/6/2010 12:10:33.264'!
test04processSnapshot

	| markup snapshot remoteClient |

	"Step 1 - Prepare a snapshot to be used."
	markup := CtMarkupGenerator new
		addTag: #talk;
		addTarget: (CtMethodSourceTarget
			class: self getTestClass
			selector: #test
			from: 1 to: 3);
		next.

	SystemMarkups addMarkup: markup.

	remoteClient := PSRemoteClient new
		sourceCode: PSSourceCode new;
		addPackage: #'CTP-Temporary'.	

	snapshot := remoteClient sourceCode snapshot.
	
	self
		assert: markup
		equals: (snapshot markups first activeMarkups at: markup uuid).
	
	(SystemMarkups instVarNamed: #activeMarkups)
		removeKey: markup uuid.
	
	"Step 2 - Create a remote client and update its snapshot."
	remoteClient := PSRemoteClient new
		sourceCode: PSSourceCode new;
		addPackage: #'CTP-Temporary'.

	PSCurrentRemoteClient value: remoteClient during: [
		self deny: ((SystemMarkups
			markupsForClass: self getTestClass
			selector: #test
			kind: #methodSource) includes: markup).
			
		remoteClient sourceCode snapshot: snapshot.

		self assert: ((SystemMarkups
			markupsForClass: self getTestClass
			selector: #test
			kind: #methodSource) includes: markup).
			
		SystemMarkups removeMarkup: markup].! !

PSSnapshot subclass: #CtPsMarkupSnapshot
	instanceVariableNames: 'defaultSnapshot markups'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'CT-PolySource-Snapshots'!

!CtPsMarkupSnapshot commentStamp: 'mt 10/4/2010 17:30:56.99' prior: 0!
Just a wrapper for a default snapshot with additional markups.!

!CtPsMarkupSnapshot methodsFor: 'loading' stamp: 'mt 10/4/2010 17:36:48.234'!
builder

	^ CtPsMarkupSnapshotBuilder new! !

!CtPsMarkupSnapshot methodsFor: 'accessing' stamp: 'mt 10/4/2010 17:30:36.087'!
defaultSnapshot

	^ defaultSnapshot! !

!CtPsMarkupSnapshot methodsFor: 'accessing' stamp: 'mt 10/4/2010 17:30:46.401'!
defaultSnapshot: aDefaultSnapshot

	defaultSnapshot := aDefaultSnapshot.! !

!CtPsMarkupSnapshot methodsFor: 'accessing' stamp: 'mt 10/4/2010 17:33:00.644'!
markups

	^ markups! !

!CtPsMarkupSnapshot methodsFor: 'accessing' stamp: 'mt 10/4/2010 17:33:09.227'!
markups: someMarkupSnapshots

	markups := someMarkupSnapshots.! !
